require 'spec_helper'
require 'timecop'

describe Time do
  describe '#sg_day' do
    context 'with US timezone' do
      let(:time_zone) { 'America/New_York' }

      it 'should return beginning_of_sg_day for standard daylight time (ET)' do
        expect(Time.parse('2014-05-10 04:00 EDT').sg_day(time_zone)).to eq(Date.parse('2014-05-09'))
        expect(Time.parse('2014-05-10 05:00 EDT').sg_day(time_zone)).to eq(Date.parse('2014-05-10'))
      end

      it 'should return beginning_of_sg_day for standard time (EST)' do
        expect(Time.parse('2014-12-10 04:00 EST').sg_day(time_zone)).to eq(Date.parse('2014-12-09'))
        expect(Time.parse('2014-12-10 05:00 EST').sg_day(time_zone)).to eq(Date.parse('2014-12-10'))
      end

      it 'should return proper sg_day for days where we switch time' do
        expect(Time.parse('2014-03-08 09:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-03-07'))
        expect(Time.parse('2014-03-08 10:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-03-08'))
        expect(Time.parse('2014-03-09 09:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-03-09'))

        expect(Time.parse('2014-11-01 08:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-10-31'))
        expect(Time.parse('2014-11-01 09:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-11-01'))
        expect(Time.parse('2014-11-01 10:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-11-01'))
        expect(Time.parse('2014-11-02 09:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-11-01'))
        expect(Time.parse('2014-11-02 10:00 UTC').sg_day(time_zone)).to eq(Date.parse('2014-11-02'))
      end
    end

    context 'with IT timezone' do
      let(:time_zone) { 'Europe/Rome' }

      it 'should return beginning_of_sg_day for CET' do
        expect(Time.parse('2014-01-10 04:00 +0100').sg_day(time_zone)).to eq(Date.parse('2014-01-09'))
        expect(Time.parse('2014-01-10 05:00 +0100').sg_day(time_zone)).to eq(Date.parse('2014-01-10'))
      end

      it 'should return beginning_of_sg_day for CEST' do
        expect(Time.parse('2014-05-10 04:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-05-09'))
        expect(Time.parse('2014-05-10 05:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-05-10'))
      end

      it 'should return proper sg_day for days where we switch from winter to summer time' do
        expect(Time.parse('2014-03-30 02:00 +0100').sg_day(time_zone)).to eq(Date.parse('2014-03-29'))
        expect(Time.parse('2014-03-30 03:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-03-29'))
        expect(Time.parse('2014-03-30 04:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-03-29'))
        expect(Time.parse('2014-03-30 05:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-03-30'))
      end

      it 'should return proper sg_day for days where we switch from summer to winter time' do
        expect(Time.parse('2014-10-26 02:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-10-25'))
        expect(Time.parse('2014-10-26 02:00 +0200').sg_day(time_zone)).to eq(Date.parse('2014-10-25'))
        expect(Time.parse('2014-10-26 03:00 +0100').sg_day(time_zone)).to eq(Date.parse('2014-10-25'))
        expect(Time.parse('2014-10-26 04:00 +0100').sg_day(time_zone)).to eq(Date.parse('2014-10-25'))
        expect(Time.parse('2014-10-26 05:00 +0100').sg_day(time_zone)).to eq(Date.parse('2014-10-26'))
      end
    end
  end

  shared_examples_for 'utc_offset' do
    it 'should get correct beginning of SG day for a timezone' do
      should == expected_offset
    end
  end

  describe '#utc_time_offset' do
    subject { time.utc_time_offset(time_zone) }

    context 'when US timezone' do
      let(:time_zone) { 'America/New_York' }

      context 'when daylight time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Time.parse('2014-06-10 00:00 UTC') }
        let(:expected_offset) { 9 }
      end

      context 'when winter time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Time.parse('2014-01-10 00:00 UTC') }
        let(:expected_offset) { 10 }
      end
    end

    context 'when IT timezone' do
      let(:time_zone) { 'Europe/Rome' }

      context 'when daylight time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Time.parse('2014-06-10 00:00 UTC') }
        let(:expected_offset) { 3 }
      end

      context 'when winter time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Time.parse('2014-01-10 00:00 UTC') }
        let(:expected_offset) { 4 }
      end
    end

    context 'when AU timezone' do
      let(:time_zone) { 'Australia/Sydney' }

      context 'when daylight time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Time.parse('2014-01-10 00:00 UTC') }
        let(:expected_offset) { -6 }
      end

      context 'when winter time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Time.parse('2014-06-10 00:00 UTC') }
        let(:expected_offset) { -5 }
      end
    end
  end

  describe '#beginning_of_sg_day' do
    let(:time_zone) { 'America/New_York' }

    it 'should raise error for non-midnight times' do
      expect { Time.parse('2013-07-08 00:10 UTC').beginning_of_sg_day('UTC') }.to raise_error(ArgumentError)
      expect { Time.parse('2013-07-08 00:00 +0200').beginning_of_sg_day('UTC') }.to raise_error(ArgumentError)
    end

    it 'should return sg offset for UTC' do
      expect(Time.parse('2013-07-08 00:00 UTC').beginning_of_sg_day('UTC')).to eq(Time.parse('2013-07-08 5:00 UTC'))
    end

    it 'should return beginning_of_sg_day for standard time (ET)' do
      expect(Time.parse('2014-03-08 00:00 UTC').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-03-08 10:00 UTC'))
    end

    it 'should return beginning_of_sg_day for daylight saving time' do
      expect(Time.parse('2014-03-09 00:00 UTC').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-03-09 09:00 UTC'))
    end

    it 'should return beginning_of_sg_day for daylight saving time' do
      expect(Time.parse('2014-11-01 00:00 UTC').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-11-01 09:00 UTC'))
    end

    it 'should return beginning_of_sg_day for standard time (ET)' do
      expect(Time.parse('2014-11-02 00:00 UTC').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-11-02 10:00 UTC'))
    end

    it 'should throw error if time is not midnight' do
      expect { Time.parse('2014-01-01 00:01 UTC').beginning_of_sg_day(time_zone) }.to raise_error(ArgumentError)
      expect { Time.parse('2014-01-01 00:00 +0200').beginning_of_sg_day(time_zone) }.to raise_error(ArgumentError)
    end
  end

  describe '#end_of_sg_day' do
    let(:time_zone) { 'America/New_York' }

    it 'should return sg offset for UTC' do
      expect(Time.parse('2013-07-08 00:00 UTC').end_of_sg_day('UTC')).to eq(Time.parse('2013-07-09 5:00 UTC'))
    end

    it 'should return end_of_sg_day for standard time (ET)' do
      expect(Time.parse('2014-03-08 00:00 UTC').end_of_sg_day(time_zone)).to eq(Time.parse('2014-03-09 09:00 UTC'))
    end

    it 'should return end_of_sg_day for daylight saving time' do
      expect(Time.parse('2014-03-09 00:00 UTC').end_of_sg_day(time_zone)).to eq(Time.parse('2014-03-10 09:00 UTC'))
    end

    it 'should return end_of_sg_day for daylight saving time' do
      expect(Time.parse('2014-11-01 00:00 UTC').end_of_sg_day(time_zone)).to eq(Time.parse('2014-11-02 10:00 UTC'))
    end

    it 'should return end_of_sg_day for standard time (ET)' do
      expect(Time.parse('2014-11-02 00:00 UTC').end_of_sg_day(time_zone)).to eq(Time.parse('2014-11-03 10:00 UTC'))
    end
  end

  describe '.current_sg_day' do
    before { Timecop.freeze(Time.parse('2015-10-01 08:00 UTC')) }
    after { Timecop.return }

    subject { Time.current_sg_day(time_zone) }

    context 'when in USA' do
      let(:time_zone) { 'America/New_York' }
      it { is_expected.to eq(Date.parse('2015-09-30 00:00 UTC')) }
    end

    context 'when in Mexico' do
      let(:time_zone) { 'America/Mexico_City' }
      it { is_expected.to eq(Date.parse('2015-09-30 00:00 UTC')) }
    end

    context 'when in Australia' do
      let(:time_zone) { 'Australia/Sydney' }
      it { is_expected.to eq(Date.parse('2015-10-01 00:00 UTC')) }
    end

    context 'when in Italy' do
      let(:time_zone) { 'Europe/Rome' }
      it { is_expected.to eq(Date.parse('2015-10-01 00:00 UTC')) }
    end
  end

  describe '.next_sg_day' do
    before { Timecop.freeze(Time.parse('2015-10-01 08:00 UTC')) }
    after { Timecop.return }

    subject { Time.next_sg_day(time_zone) }

    context 'when in USA' do
      let(:time_zone) { 'America/New_York' }
      it { is_expected.to eq(Date.parse('2015-10-01 00:00 UTC')) }
    end

    context 'when in Mexico' do
      let(:time_zone) { 'America/Mexico_City' }
      it { is_expected.to eq(Date.parse('2015-10-01 00:00 UTC')) }
    end

    context 'when in Australia' do
      let(:time_zone) { 'Australia/Sydney' }
      it { is_expected.to eq(Date.parse('2015-10-02 00:00 UTC')) }
    end

    context 'when in Italy' do
      let(:time_zone) { 'Europe/Rome' }
      it { is_expected.to eq(Date.parse('2015-10-02 00:00 UTC')) }
    end
  end
end
